/* forward_exs.h */

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grant No. OCI-1127228.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

/*
 * The UNH IOL implementation of the Extended Sockets API (ES-API) Issue 1.0
 *
 * defined in:	The Interconnect Software Consortium Issue 1.0
 *		ISBN: 1-931624-52-6
 *		Published by The open Group, January 2005.
 *
 * all page references in this source code are to pages in that document.
 */

/*
 * Most of the coding in this file done by:
 *	Bob Russell
 *	rdr@iol.unh.edu
 */


/* collect all forward declarations here at the beginning */
static void
our_free_qp(struct exs_conn *cb);

static int
our_setup_qp(struct exs_conn *cb, struct rdma_cm_id *cm_id);

static void
our_free_transaction_block(struct transaction_block *block);

static int
our_merge_pending_accept(struct exs_conn *listen_conn,
			 struct rdma_cm_event *cm_event);

static int
our_finish_pending_accept(struct exs_conn *listen_conn);

static int
our_connect_returning(struct exs_conn *new_conn, int err_level, int ret,
			struct evt_queue **qhandle, int *signo);

static void
our_connect_done(struct exs_conn *new_conn,
			struct evt_queue **qhandle, int *signo);

static int
our_connect_routing(struct exs_conn *new_conn, struct evt_queue **qhandle,
			int *signo);

static int
our_connect_resolving(struct exs_conn *new_conn, struct evt_queue **qhandle,
			int *signo);

static int
our_connect_finishing(struct exs_conn *new_conn, struct rdma_cm_event *event,
			struct evt_queue **qhandle, int *signo);

static void
our_close_connecting(struct exs_conn *this_conn, const char *message,
			 struct evt_queue **qhandle, int *signo);

static void
our_undo_exs_socket(int fd, struct exs_conn *this_conn, int level,
			const char *message, struct evt_queue **qhandle,
			int *signo);

static int
our_rdma_disconnect(struct exs_conn *this_conn, const char *message);

static void
our_free_all_pending_lists(struct exs_conn *this_conn, const char *message);


static void
our_flush_stranded_send_blocks(struct exs_conn *this_conn, int errcode);


static void
our_flush_all_stranded_recv_blocks(struct exs_conn *this_conn);


static int
our_xmit_if_match_seqpacket(struct exs_conn *this_conn,
			struct transaction_block *block,
			our_qhandle_t *qhandle, int *signo,
			const char *message);

static int
our_xmit_if_match_stream(struct exs_conn *this_conn,
			struct transaction_block *block,
			our_qhandle_t *qhandle, int *signo,
			const char *message);

static void
print_send_wr(FILE *fptr, struct ibv_send_wr *wr);

static struct transaction_block *
our_init_transaction_block(struct exs_conn *cb, uint32_t sp_size,
		uint32_t copy_unreg_on_send, struct transaction_block *block,
		struct ibv_mr *mr);

static void
our_uninit_transaction_block(struct transaction_block *block);

static int
our_enqueue_send_seqpacket(struct exs_conn *this_conn, struct transaction_block *this,
		const void *buffer, size_t length, our_qhandle_t *qptr,
		int *signo);

static int
our_enqueue_send_stream(struct exs_conn *this_conn, struct transaction_block *this,
		const void *buffer, size_t length, our_qhandle_t *qptr,
		int *signo);

static int
our_handle_recv_advert_seqpacket(struct exs_conn *cb,
				struct transaction_block *block,
				struct ibv_wc *wc,
				char *errbuf, const char *message,
				int *do_eof_broadcast, int *force_exit,
				our_qhandle_t *qhandle, int *signo);

static int
our_handle_recv_advert_stream(struct exs_conn *cb,
				struct transaction_block *block,
				struct ibv_wc *wc,
				char *errbuf, const char *message,
				int *do_eof_broadcast, int *force_exit,
				our_qhandle_t *qhandle, int *signo);

static int
our_handle_send_write_seqpacket(struct exs_conn *cb,
		struct transaction_block *block, const char *message,
		our_qhandle_t *qhandle, int *signo);

static int
our_handle_send_write_stream(struct exs_conn *cb,
		struct transaction_block *block, const char *message,
		our_qhandle_t *qhandle, int *signo);

static int
our_send_direct(struct exs_conn *cb, struct transaction_block *block);
